// Copyright 2026 Google LLC.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Code generated file. DO NOT EDIT.

// Package chromeuxreport provides access to the Chrome UX Report API.
//
// For product documentation, see: https://developers.google.com/web/tools/chrome-user-experience-report/api/reference
//
// # Library status
//
// These client libraries are officially supported by Google. However, this
// library is considered complete and is in maintenance mode. This means
// that we will address critical bugs and security issues but will not add
// any new features.
//
// When possible, we recommend using our newer
// [Cloud Client Libraries for Go](https://pkg.go.dev/cloud.google.com/go)
// that are still actively being worked and iterated on.
//
// # Creating a client
//
// Usage example:
//
//	import "google.golang.org/api/chromeuxreport/v1"
//	...
//	ctx := context.Background()
//	chromeuxreportService, err := chromeuxreport.NewService(ctx)
//
// In this example, Google Application Default Credentials are used for
// authentication. For information on how to create and obtain Application
// Default Credentials, see https://developers.google.com/identity/protocols/application-default-credentials.
//
// # Other authentication options
//
// To use an API key for authentication (note: some APIs do not support API
// keys), use [google.golang.org/api/option.WithAPIKey]:
//
//	chromeuxreportService, err := chromeuxreport.NewService(ctx, option.WithAPIKey("AIza..."))
//
// To use an OAuth token (e.g., a user token obtained via a three-legged OAuth
// flow, use [google.golang.org/api/option.WithTokenSource]:
//
//	config := &oauth2.Config{...}
//	// ...
//	token, err := config.Exchange(ctx, ...)
//	chromeuxreportService, err := chromeuxreport.NewService(ctx, option.WithTokenSource(config.TokenSource(ctx, token)))
//
// See [google.golang.org/api/option.ClientOption] for details on options.
package chromeuxreport // import "google.golang.org/api/chromeuxreport/v1"

import (
	"bytes"
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"net/url"
	"strconv"
	"strings"

	"github.com/googleapis/gax-go/v2/internallog"
	googleapi "google.golang.org/api/googleapi"
	internal "google.golang.org/api/internal"
	gensupport "google.golang.org/api/internal/gensupport"
	option "google.golang.org/api/option"
	internaloption "google.golang.org/api/option/internaloption"
	htransport "google.golang.org/api/transport/http"
)

// Always reference these packages, just in case the auto-generated code
// below doesn't.
var _ = bytes.NewBuffer
var _ = strconv.Itoa
var _ = fmt.Sprintf
var _ = json.NewDecoder
var _ = io.Copy
var _ = url.Parse
var _ = gensupport.MarshalJSON
var _ = googleapi.Version
var _ = errors.New
var _ = strings.Replace
var _ = context.Canceled
var _ = internaloption.WithDefaultEndpoint
var _ = internal.Version
var _ = internallog.New

const apiId = "chromeuxreport:v1"
const apiName = "chromeuxreport"
const apiVersion = "v1"
const basePath = "https://chromeuxreport.googleapis.com/"
const basePathTemplate = "https://chromeuxreport.UNIVERSE_DOMAIN/"
const mtlsBasePath = "https://chromeuxreport.mtls.googleapis.com/"

// NewService creates a new Service.
func NewService(ctx context.Context, opts ...option.ClientOption) (*Service, error) {
	opts = append(opts, internaloption.WithDefaultEndpoint(basePath))
	opts = append(opts, internaloption.WithDefaultEndpointTemplate(basePathTemplate))
	opts = append(opts, internaloption.WithDefaultMTLSEndpoint(mtlsBasePath))
	opts = append(opts, internaloption.EnableNewAuthLibrary())
	client, endpoint, err := htransport.NewClient(ctx, opts...)
	if err != nil {
		return nil, err
	}
	s := &Service{client: client, BasePath: basePath, logger: internaloption.GetLogger(opts)}
	s.Records = NewRecordsService(s)
	if endpoint != "" {
		s.BasePath = endpoint
	}
	return s, nil
}

// New creates a new Service. It uses the provided http.Client for requests.
//
// Deprecated: please use NewService instead.
// To provide a custom HTTP client, use option.WithHTTPClient.
// If you are using google.golang.org/api/googleapis/transport.APIKey, use option.WithAPIKey with NewService instead.
func New(client *http.Client) (*Service, error) {
	if client == nil {
		return nil, errors.New("client is nil")
	}
	return NewService(context.TODO(), option.WithHTTPClient(client))
}

type Service struct {
	client    *http.Client
	logger    *slog.Logger
	BasePath  string // API endpoint base URL
	UserAgent string // optional additional User-Agent fragment

	Records *RecordsService
}

func (s *Service) userAgent() string {
	if s.UserAgent == "" {
		return googleapi.UserAgent
	}
	return googleapi.UserAgent + " " + s.UserAgent
}

func NewRecordsService(s *Service) *RecordsService {
	rs := &RecordsService{s: s}
	return rs
}

type RecordsService struct {
	s *Service
}

// Bin: A bin is a discrete portion of data spanning from start to end, or if
// no end is given, then from start to +inf. A bin's start and end values are
// given in the value type of the metric it represents. For example, "first
// contentful paint" is measured in milliseconds and exposed as ints, therefore
// its metric bins will use int32s for its start and end types. However,
// "cumulative layout shift" is measured in unitless decimals and is exposed as
// a decimal encoded as a string, therefore its metric bins will use strings
// for its value type.
type Bin struct {
	// Density: The proportion of users that experienced this bin's value for the
	// given metric.
	Density interface{} `json:"density,omitempty"`
	// End: End is the end of the data bin. If end is not populated, then the bin
	// has no end and is valid from start to +inf.
	End interface{} `json:"end,omitempty"`
	// Start: Start is the beginning of the data bin.
	Start interface{} `json:"start,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Density") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Density") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s Bin) MarshalJSON() ([]byte, error) {
	type NoMethod Bin
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// CollectionPeriod: The collection period is a date range which includes the
// `first` and `last` day.
type CollectionPeriod struct {
	// FirstDate: The first day in the collection period, inclusive.
	FirstDate *Date `json:"firstDate,omitempty"`
	// LastDate: The last day in the collection period, inclusive.
	LastDate *Date `json:"lastDate,omitempty"`
	// ForceSendFields is a list of field names (e.g. "FirstDate") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "FirstDate") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s CollectionPeriod) MarshalJSON() ([]byte, error) {
	type NoMethod CollectionPeriod
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// Date: Represents a whole or partial calendar date, such as a birthday. The
// time of day and time zone are either specified elsewhere or are
// insignificant. The date is relative to the Gregorian Calendar. This can
// represent one of the following: * A full date, with non-zero year, month,
// and day values. * A month and day, with a zero year (for example, an
// anniversary). * A year on its own, with a zero month and a zero day. * A
// year and month, with a zero day (for example, a credit card expiration
// date). Related types: * google.type.TimeOfDay * google.type.DateTime *
// google.protobuf.Timestamp
type Date struct {
	// Day: Day of a month. Must be from 1 to 31 and valid for the year and month,
	// or 0 to specify a year by itself or a year and month where the day isn't
	// significant.
	Day int64 `json:"day,omitempty"`
	// Month: Month of a year. Must be from 1 to 12, or 0 to specify a year without
	// a month and day.
	Month int64 `json:"month,omitempty"`
	// Year: Year of the date. Must be from 1 to 9999, or 0 to specify a date
	// without a year.
	Year int64 `json:"year,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Day") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Day") to include in API requests
	// with the JSON null value. By default, fields with empty values are omitted
	// from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s Date) MarshalJSON() ([]byte, error) {
	type NoMethod Date
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// FractionTimeseries: For enum metrics, provides fraction timeseries which add
// up to approximately 1.0 per entry (k-th element into the repeated fractions
// field for any k <= len) across fraction_timeseries.
type FractionTimeseries struct {
	// Fractions: Values between 0.0 and 1.0 (inclusive) and NaN.
	Fractions []float64 `json:"fractions,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Fractions") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Fractions") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s FractionTimeseries) MarshalJSON() ([]byte, error) {
	type NoMethod FractionTimeseries
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

func (s *FractionTimeseries) UnmarshalJSON(data []byte) error {
	type NoMethod FractionTimeseries
	var s1 struct {
		Fractions []gensupport.JSONFloat64 `json:"fractions"`
		*NoMethod
	}
	s1.NoMethod = (*NoMethod)(s)
	if err := json.Unmarshal(data, &s1); err != nil {
		return err
	}
	s.Fractions = make([]float64, len(s1.Fractions))
	for i := range s1.Fractions {
		s.Fractions[i] = float64(s1.Fractions[i])
	}
	return nil
}

// HistoryKey: Key defines all the dimensions that identify this record as
// unique.
type HistoryKey struct {
	// FormFactor: The form factor is the device class that all users used to
	// access the site for this record. If the form factor is unspecified, then
	// aggregated data over all form factors will be returned.
	//
	// Possible values:
	//   "ALL_FORM_FACTORS" - The default value, representing all device classes.
	//   "PHONE" - The device class representing a "mobile"/"phone" sized client.
	//   "DESKTOP" - The device class representing a "desktop"/"laptop" type full
	// size client.
	//   "TABLET" - The device class representing a "tablet" type client.
	FormFactor string `json:"formFactor,omitempty"`
	// Origin: Origin specifies the origin that this record is for. Note: When
	// specifying an origin, data for loads under this origin over all pages are
	// aggregated into origin level user experience data.
	Origin string `json:"origin,omitempty"`
	// Url: Url specifies a specific url that this record is for. This url should
	// be normalized, following the normalization actions taken in the request to
	// increase the chances of successful lookup. Note: When specifying a "url"
	// only data for that specific url will be aggregated.
	Url string `json:"url,omitempty"`
	// ForceSendFields is a list of field names (e.g. "FormFactor") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "FormFactor") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s HistoryKey) MarshalJSON() ([]byte, error) {
	type NoMethod HistoryKey
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// HistoryRecord: HistoryRecord is a timeseries of Chrome UX Report data. It
// contains user experience statistics for a single url pattern and a set of
// dimensions.
type HistoryRecord struct {
	// CollectionPeriods: The collection periods indicate when each of the data
	// points reflected in the time series data in metrics was collected. Note that
	// all the time series share the same collection periods, and it is enforced in
	// the CrUX pipeline that every time series has the same number of data points.
	CollectionPeriods []*CollectionPeriod `json:"collectionPeriods,omitempty"`
	// Key: Key defines all of the unique querying parameters needed to look up a
	// user experience history record.
	Key *HistoryKey `json:"key,omitempty"`
	// Metrics: Metrics is the map of user experience time series data available
	// for the record defined in the key field. Metrics are keyed on the metric
	// name. Allowed key values: ["first_contentful_paint", "first_input_delay",
	// "largest_contentful_paint", "cumulative_layout_shift",
	// "experimental_time_to_first_byte", "experimental_interaction_to_next_paint"]
	Metrics map[string]MetricTimeseries `json:"metrics,omitempty"`
	// ForceSendFields is a list of field names (e.g. "CollectionPeriods") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "CollectionPeriods") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s HistoryRecord) MarshalJSON() ([]byte, error) {
	type NoMethod HistoryRecord
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// Key: Key defines all the dimensions that identify this record as unique.
type Key struct {
	// EffectiveConnectionType: The effective connection type is the general
	// connection class that all users experienced for this record. This field uses
	// the values ["offline", "slow-2G", "2G", "3G", "4G"] as specified in:
	// https://wicg.github.io/netinfo/#effective-connection-types If the effective
	// connection type is unspecified, then aggregated data over all effective
	// connection types will be returned.
	EffectiveConnectionType string `json:"effectiveConnectionType,omitempty"`
	// FormFactor: The form factor is the device class that all users used to
	// access the site for this record. If the form factor is unspecified, then
	// aggregated data over all form factors will be returned.
	//
	// Possible values:
	//   "ALL_FORM_FACTORS" - The default value, representing all device classes.
	//   "PHONE" - The device class representing a "mobile"/"phone" sized client.
	//   "DESKTOP" - The device class representing a "desktop"/"laptop" type full
	// size client.
	//   "TABLET" - The device class representing a "tablet" type client.
	FormFactor string `json:"formFactor,omitempty"`
	// Origin: Origin specifies the origin that this record is for. Note: When
	// specifying an origin, data for loads under this origin over all pages are
	// aggregated into origin level user experience data.
	Origin string `json:"origin,omitempty"`
	// Url: Url specifies a specific url that this record is for. Note: When
	// specifying a "url" only data for that specific url will be aggregated.
	Url string `json:"url,omitempty"`
	// ForceSendFields is a list of field names (e.g. "EffectiveConnectionType") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "EffectiveConnectionType") to
	// include in API requests with the JSON null value. By default, fields with
	// empty values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s Key) MarshalJSON() ([]byte, error) {
	type NoMethod Key
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// Metric: A `metric` is a set of user experience data for a single web
// performance metric, like "first contentful paint". It contains a summary
// histogram of real world Chrome usage as a series of `bins`.
type Metric struct {
	// Fractions: For enum metrics, provides fractions which add up to
	// approximately 1.0.
	Fractions map[string]float64 `json:"fractions,omitempty"`
	// Histogram: The histogram of user experiences for a metric. The histogram
	// will have at least one bin and the densities of all bins will add up to ~1.
	Histogram []*Bin `json:"histogram,omitempty"`
	// Percentiles: Commonly useful percentiles of the Metric. The value type for
	// the percentiles will be the same as the value types given for the Histogram
	// bins.
	Percentiles *Percentiles `json:"percentiles,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Fractions") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Fractions") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s Metric) MarshalJSON() ([]byte, error) {
	type NoMethod Metric
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// MetricTimeseries: A `metric timeseries` is a set of user experience data for
// a single web performance metric, like "first contentful paint". It contains
// a summary histogram of real world Chrome usage as a series of `bins`, where
// each bin has density values for a particular time period.
type MetricTimeseries struct {
	// FractionTimeseries: Mapping from labels to timeseries of fractions
	// attributed to this label.
	FractionTimeseries map[string]FractionTimeseries `json:"fractionTimeseries,omitempty"`
	// HistogramTimeseries: The histogram of user experiences for a metric. The
	// histogram will have at least one bin and the densities of all bins will add
	// up to ~1, for each timeseries entry.
	HistogramTimeseries []*TimeseriesBin `json:"histogramTimeseries,omitempty"`
	// PercentilesTimeseries: Commonly useful percentiles of the Metric. The value
	// type for the percentiles will be the same as the value types given for the
	// Histogram bins.
	PercentilesTimeseries *TimeseriesPercentiles `json:"percentilesTimeseries,omitempty"`
	// ForceSendFields is a list of field names (e.g. "FractionTimeseries") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "FractionTimeseries") to include
	// in API requests with the JSON null value. By default, fields with empty
	// values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s MetricTimeseries) MarshalJSON() ([]byte, error) {
	type NoMethod MetricTimeseries
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// Percentiles: Percentiles contains synthetic values of a metric at a given
// statistical percentile. These are used for estimating a metric's value as
// experienced by a percentage of users out of the total number of users.
type Percentiles struct {
	// P75: 75% of users experienced the given metric at or below this value.
	P75 interface{} `json:"p75,omitempty"`
	// ForceSendFields is a list of field names (e.g. "P75") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "P75") to include in API requests
	// with the JSON null value. By default, fields with empty values are omitted
	// from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s Percentiles) MarshalJSON() ([]byte, error) {
	type NoMethod Percentiles
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// QueryHistoryRequest: Request payload sent by a physical web client. This
// request includes all necessary context to load a particular user experience
// history record.
type QueryHistoryRequest struct {
	// CollectionPeriodCount: The number of collection periods to return. If not
	// specified, the default is 25. If present, must be in the range [1, 40].
	CollectionPeriodCount int64 `json:"collectionPeriodCount,omitempty"`
	// FormFactor: The form factor is a query dimension that specifies the device
	// class that the record's data should belong to. Note: If no form factor is
	// specified, then a special record with aggregated data over all form factors
	// will be returned.
	//
	// Possible values:
	//   "ALL_FORM_FACTORS" - The default value, representing all device classes.
	//   "PHONE" - The device class representing a "mobile"/"phone" sized client.
	//   "DESKTOP" - The device class representing a "desktop"/"laptop" type full
	// size client.
	//   "TABLET" - The device class representing a "tablet" type client.
	FormFactor string `json:"formFactor,omitempty"`
	// Metrics: The metrics that should be included in the response. If none are
	// specified then any metrics found will be returned. Allowed values:
	// ["first_contentful_paint", "first_input_delay", "largest_contentful_paint",
	// "cumulative_layout_shift", "experimental_time_to_first_byte",
	// "experimental_interaction_to_next_paint"]
	Metrics []string `json:"metrics,omitempty"`
	// Origin: The url pattern "origin" refers to a url pattern that is the origin
	// of a website. Examples: "https://example.com", "https://cloud.google.com"
	Origin string `json:"origin,omitempty"`
	// Url: The url pattern "url" refers to a url pattern that is any arbitrary
	// url. Examples: "https://example.com/",
	// "https://cloud.google.com/why-google-cloud/"
	Url string `json:"url,omitempty"`
	// ForceSendFields is a list of field names (e.g. "CollectionPeriodCount") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "CollectionPeriodCount") to
	// include in API requests with the JSON null value. By default, fields with
	// empty values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s QueryHistoryRequest) MarshalJSON() ([]byte, error) {
	type NoMethod QueryHistoryRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// QueryHistoryResponse: Response payload sent back to a physical web client.
// This response contains the record found based on the identiers present in a
// `QueryHistoryRequest`. The returned response will have a history record, and
// sometimes details on normalization actions taken on the request that were
// necessary to make the request successful.
type QueryHistoryResponse struct {
	// Record: The record that was found.
	Record *HistoryRecord `json:"record,omitempty"`
	// UrlNormalizationDetails: These are details about automated normalization
	// actions that were taken in order to make the requested `url_pattern` valid.
	UrlNormalizationDetails *UrlNormalization `json:"urlNormalizationDetails,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "Record") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Record") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s QueryHistoryResponse) MarshalJSON() ([]byte, error) {
	type NoMethod QueryHistoryResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// QueryRequest: Request payload sent by a physical web client. This request
// includes all necessary context to load a particular user experience record.
type QueryRequest struct {
	// EffectiveConnectionType: The effective connection type is a query dimension
	// that specifies the effective network class that the record's data should
	// belong to. This field uses the values ["offline", "slow-2G", "2G", "3G",
	// "4G"] as specified in:
	// https://wicg.github.io/netinfo/#effective-connection-types Note: If no
	// effective connection type is specified, then a special record with
	// aggregated data over all effective connection types will be returned.
	EffectiveConnectionType string `json:"effectiveConnectionType,omitempty"`
	// FormFactor: The form factor is a query dimension that specifies the device
	// class that the record's data should belong to. Note: If no form factor is
	// specified, then a special record with aggregated data over all form factors
	// will be returned.
	//
	// Possible values:
	//   "ALL_FORM_FACTORS" - The default value, representing all device classes.
	//   "PHONE" - The device class representing a "mobile"/"phone" sized client.
	//   "DESKTOP" - The device class representing a "desktop"/"laptop" type full
	// size client.
	//   "TABLET" - The device class representing a "tablet" type client.
	FormFactor string `json:"formFactor,omitempty"`
	// Metrics: The metrics that should be included in the response. If none are
	// specified then any metrics found will be returned. Allowed values:
	// ["first_contentful_paint", "first_input_delay", "largest_contentful_paint",
	// "cumulative_layout_shift", "experimental_time_to_first_byte",
	// "experimental_interaction_to_next_paint"]
	Metrics []string `json:"metrics,omitempty"`
	// Origin: The url pattern "origin" refers to a url pattern that is the origin
	// of a website. Examples: "https://example.com", "https://cloud.google.com"
	Origin string `json:"origin,omitempty"`
	// Url: The url pattern "url" refers to a url pattern that is any arbitrary
	// url. Examples: "https://example.com/",
	// "https://cloud.google.com/why-google-cloud/"
	Url string `json:"url,omitempty"`
	// ForceSendFields is a list of field names (e.g. "EffectiveConnectionType") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "EffectiveConnectionType") to
	// include in API requests with the JSON null value. By default, fields with
	// empty values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s QueryRequest) MarshalJSON() ([]byte, error) {
	type NoMethod QueryRequest
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// QueryResponse: Response payload sent back to a physical web client. This
// response contains the record found based on the identiers present in a
// `QueryRequest`. The returned response will have a record, and sometimes
// details on normalization actions taken on the request that were necessary to
// make the request successful.
type QueryResponse struct {
	// Record: The record that was found.
	Record *Record `json:"record,omitempty"`
	// UrlNormalizationDetails: These are details about automated normalization
	// actions that were taken in order to make the requested `url_pattern` valid.
	UrlNormalizationDetails *UrlNormalization `json:"urlNormalizationDetails,omitempty"`

	// ServerResponse contains the HTTP response code and headers from the server.
	googleapi.ServerResponse `json:"-"`
	// ForceSendFields is a list of field names (e.g. "Record") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Record") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s QueryResponse) MarshalJSON() ([]byte, error) {
	type NoMethod QueryResponse
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// Record: Record is a single Chrome UX report data record. It contains use
// experience statistics for a single url pattern and set of dimensions.
type Record struct {
	// CollectionPeriod: The collection period indicates when the data reflected in
	// this record was collected.
	CollectionPeriod *CollectionPeriod `json:"collectionPeriod,omitempty"`
	// Key: Key defines all of the unique querying parameters needed to look up a
	// user experience record.
	Key *Key `json:"key,omitempty"`
	// Metrics: Metrics is the map of user experience data available for the record
	// defined in the key field. Metrics are keyed on the metric name. Allowed key
	// values: ["first_contentful_paint", "first_input_delay",
	// "largest_contentful_paint", "cumulative_layout_shift",
	// "experimental_time_to_first_byte", "experimental_interaction_to_next_paint"]
	Metrics map[string]Metric `json:"metrics,omitempty"`
	// ForceSendFields is a list of field names (e.g. "CollectionPeriod") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "CollectionPeriod") to include in
	// API requests with the JSON null value. By default, fields with empty values
	// are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s Record) MarshalJSON() ([]byte, error) {
	type NoMethod Record
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// TimeseriesBin: A bin is a discrete portion of data spanning from start to
// end, or if no end is given, then from start to +inf. A bin's start and end
// values are given in the value type of the metric it represents. For example,
// "first contentful paint" is measured in milliseconds and exposed as ints,
// therefore its metric bins will use int32s for its start and end types.
// However, "cumulative layout shift" is measured in unitless decimals and is
// exposed as a decimal encoded as a string, therefore its metric bins will use
// strings for its value type.
type TimeseriesBin struct {
	// Densities: The proportion of users that experienced this bin's value for the
	// given metric in a given collection period; the index for each of these
	// entries corresponds to an entry in the CollectionPeriods field in the
	// HistoryRecord message, which describes when the density was observed in the
	// field. Thus, the length of this list of densities is equal to the length of
	// the CollectionPeriods field in the HistoryRecord message.
	Densities []float64 `json:"densities,omitempty"`
	// End: End is the end of the data bin. If end is not populated, then the bin
	// has no end and is valid from start to +inf.
	End interface{} `json:"end,omitempty"`
	// Start: Start is the beginning of the data bin.
	Start interface{} `json:"start,omitempty"`
	// ForceSendFields is a list of field names (e.g. "Densities") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "Densities") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s TimeseriesBin) MarshalJSON() ([]byte, error) {
	type NoMethod TimeseriesBin
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

func (s *TimeseriesBin) UnmarshalJSON(data []byte) error {
	type NoMethod TimeseriesBin
	var s1 struct {
		Densities []gensupport.JSONFloat64 `json:"densities"`
		*NoMethod
	}
	s1.NoMethod = (*NoMethod)(s)
	if err := json.Unmarshal(data, &s1); err != nil {
		return err
	}
	s.Densities = make([]float64, len(s1.Densities))
	for i := range s1.Densities {
		s.Densities[i] = float64(s1.Densities[i])
	}
	return nil
}

// TimeseriesPercentiles: Percentiles contains synthetic values of a metric at
// a given statistical percentile. These are used for estimating a metric's
// value as experienced by a percentage of users out of the total number of
// users.
type TimeseriesPercentiles struct {
	// P75s: 75% of users experienced the given metric at or below this value. The
	// length of this list of densities is equal to the length of the
	// CollectionPeriods field in the HistoryRecord message, which describes when
	// the density was observed in the field.
	P75s []interface{} `json:"p75s,omitempty"`
	// ForceSendFields is a list of field names (e.g. "P75s") to unconditionally
	// include in API requests. By default, fields with empty or default values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "P75s") to include in API requests
	// with the JSON null value. By default, fields with empty values are omitted
	// from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s TimeseriesPercentiles) MarshalJSON() ([]byte, error) {
	type NoMethod TimeseriesPercentiles
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

// UrlNormalization: Object representing the normalization actions taken to
// normalize a url to achieve a higher chance of successful lookup. These are
// simple automated changes that are taken when looking up the provided
// `url_patten` would be known to fail. Complex actions like following
// redirects are not handled.
type UrlNormalization struct {
	// NormalizedUrl: The URL after any normalization actions. This is a valid user
	// experience URL that could reasonably be looked up.
	NormalizedUrl string `json:"normalizedUrl,omitempty"`
	// OriginalUrl: The original requested URL prior to any normalization actions.
	OriginalUrl string `json:"originalUrl,omitempty"`
	// ForceSendFields is a list of field names (e.g. "NormalizedUrl") to
	// unconditionally include in API requests. By default, fields with empty or
	// default values are omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-ForceSendFields for more
	// details.
	ForceSendFields []string `json:"-"`
	// NullFields is a list of field names (e.g. "NormalizedUrl") to include in API
	// requests with the JSON null value. By default, fields with empty values are
	// omitted from API requests. See
	// https://pkg.go.dev/google.golang.org/api#hdr-NullFields for more details.
	NullFields []string `json:"-"`
}

func (s UrlNormalization) MarshalJSON() ([]byte, error) {
	type NoMethod UrlNormalization
	return gensupport.MarshalJSON(NoMethod(s), s.ForceSendFields, s.NullFields)
}

type RecordsQueryHistoryRecordCall struct {
	s                   *Service
	queryhistoryrequest *QueryHistoryRequest
	urlParams_          gensupport.URLParams
	ctx_                context.Context
	header_             http.Header
}

// QueryHistoryRecord: Queries the Chrome User Experience Report for a
// timeseries `history record` for a given site. Returns a `history record`
// that contains one or more `metric timeseries` corresponding to performance
// data about the requested site.
func (r *RecordsService) QueryHistoryRecord(queryhistoryrequest *QueryHistoryRequest) *RecordsQueryHistoryRecordCall {
	c := &RecordsQueryHistoryRecordCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.queryhistoryrequest = queryhistoryrequest
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *RecordsQueryHistoryRecordCall) Fields(s ...googleapi.Field) *RecordsQueryHistoryRecordCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *RecordsQueryHistoryRecordCall) Context(ctx context.Context) *RecordsQueryHistoryRecordCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *RecordsQueryHistoryRecordCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *RecordsQueryHistoryRecordCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.queryhistoryrequest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1/records:queryHistoryRecord")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "chromeuxreport.records.queryHistoryRecord", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "chromeuxreport.records.queryHistoryRecord" call.
// Any non-2xx status code is an error. Response headers are in either
// *QueryHistoryResponse.ServerResponse.Header or (if a response was returned
// at all) in error.(*googleapi.Error).Header. Use googleapi.IsNotModified to
// check whether the returned error was because http.StatusNotModified was
// returned.
func (c *RecordsQueryHistoryRecordCall) Do(opts ...googleapi.CallOption) (*QueryHistoryResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &QueryHistoryResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "chromeuxreport.records.queryHistoryRecord", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}

type RecordsQueryRecordCall struct {
	s            *Service
	queryrequest *QueryRequest
	urlParams_   gensupport.URLParams
	ctx_         context.Context
	header_      http.Header
}

// QueryRecord: Queries the Chrome User Experience for a single `record` for a
// given site. Returns a `record` that contains one or more `metrics`
// corresponding to performance data about the requested site.
func (r *RecordsService) QueryRecord(queryrequest *QueryRequest) *RecordsQueryRecordCall {
	c := &RecordsQueryRecordCall{s: r.s, urlParams_: make(gensupport.URLParams)}
	c.queryrequest = queryrequest
	return c
}

// Fields allows partial responses to be retrieved. See
// https://developers.google.com/gdata/docs/2.0/basics#PartialResponse for more
// details.
func (c *RecordsQueryRecordCall) Fields(s ...googleapi.Field) *RecordsQueryRecordCall {
	c.urlParams_.Set("fields", googleapi.CombineFields(s))
	return c
}

// Context sets the context to be used in this call's Do method.
func (c *RecordsQueryRecordCall) Context(ctx context.Context) *RecordsQueryRecordCall {
	c.ctx_ = ctx
	return c
}

// Header returns a http.Header that can be modified by the caller to add
// headers to the request.
func (c *RecordsQueryRecordCall) Header() http.Header {
	if c.header_ == nil {
		c.header_ = make(http.Header)
	}
	return c.header_
}

func (c *RecordsQueryRecordCall) doRequest(alt string) (*http.Response, error) {
	reqHeaders := gensupport.SetHeaders(c.s.userAgent(), "application/json", c.header_)
	body, err := googleapi.WithoutDataWrapper.JSONBuffer(c.queryrequest)
	if err != nil {
		return nil, err
	}
	c.urlParams_.Set("alt", alt)
	c.urlParams_.Set("prettyPrint", "false")
	urls := googleapi.ResolveRelative(c.s.BasePath, "v1/records:queryRecord")
	urls += "?" + c.urlParams_.Encode()
	req, err := http.NewRequest("POST", urls, body)
	if err != nil {
		return nil, err
	}
	req.Header = reqHeaders
	c.s.logger.DebugContext(c.ctx_, "api request", "serviceName", apiName, "rpcName", "chromeuxreport.records.queryRecord", "request", internallog.HTTPRequest(req, body.Bytes()))
	return gensupport.SendRequest(c.ctx_, c.s.client, req)
}

// Do executes the "chromeuxreport.records.queryRecord" call.
// Any non-2xx status code is an error. Response headers are in either
// *QueryResponse.ServerResponse.Header or (if a response was returned at all)
// in error.(*googleapi.Error).Header. Use googleapi.IsNotModified to check
// whether the returned error was because http.StatusNotModified was returned.
func (c *RecordsQueryRecordCall) Do(opts ...googleapi.CallOption) (*QueryResponse, error) {
	gensupport.SetOptions(c.urlParams_, opts...)
	res, err := c.doRequest("json")
	if res != nil && res.StatusCode == http.StatusNotModified {
		if res.Body != nil {
			res.Body.Close()
		}
		return nil, gensupport.WrapError(&googleapi.Error{
			Code:   res.StatusCode,
			Header: res.Header,
		})
	}
	if err != nil {
		return nil, err
	}
	defer googleapi.CloseBody(res)
	if err := googleapi.CheckResponse(res); err != nil {
		return nil, gensupport.WrapError(err)
	}
	ret := &QueryResponse{
		ServerResponse: googleapi.ServerResponse{
			Header:         res.Header,
			HTTPStatusCode: res.StatusCode,
		},
	}
	target := &ret
	b, err := gensupport.DecodeResponseBytes(target, res)
	if err != nil {
		return nil, err
	}
	c.s.logger.DebugContext(c.ctx_, "api response", "serviceName", apiName, "rpcName", "chromeuxreport.records.queryRecord", "response", internallog.HTTPResponse(res, b))
	return ret, nil
}
